@@ -367,11 +367,11 @@ class RecurringComponent(Component):
                     rruleset = dateutil.rrule.rruleset()
                 if addfunc is None:
                     addfunc=getattr(rruleset, name)
-                
+
                 if name in DATENAMES:
                     if type(line.value[0]) == datetime.datetime:
                         map(addfunc, line.value)
-                    elif type(line.value) == datetime.date:
+                    elif type(line.value[0]) == datetime.date:
                         for dt in line.value:
                             addfunc(datetime.datetime(dt.year, dt.month, dt.day))
                     else:
@@ -404,11 +404,14 @@ class RecurringComponent(Component):
                         vals = dict(pair.split('=') for pair in
                                     line.value.upper().split(';'))
                         if len(vals.get('UNTIL', '')) == 8:
-                            # it's not entirely clear, but presumably a date
-                            # valued UNTIL should include that date
                             until = datetime.datetime.combine(until.date(),
                                                               dtstart.time())
-                        rule._until = until.replace(tzinfo=dtstart.tzinfo)
+                            until = until.replace(tzinfo=dtstart.tzinfo)
+
+                        if dtstart.tzinfo is not None:
+                            until = until.astimezone(dtstart.tzinfo)
+
+                        rule._until = until
                     
                     # add the rrule or exrule to the rruleset
                     addfunc(rule)
@@ -1458,11 +1461,14 @@ def stringToDateTime(s, tzinfo=None):
 
 escapableCharList = "\\;,Nn"
 
-def stringToTextValues(s, strict=False):
+def stringToTextValues(s, listSeparator=',', charList=None, strict=False):
     """Returns list of strings."""
+    
+    if charList is None:
+        charList = escapableCharList
 
     def escapableChar (c):
-        return c in escapableCharList
+        return c in charList
 
     def error(msg):
         if strict:
@@ -1487,7 +1493,7 @@ def stringToTextValues(s, strict=False):
         if state == "read normal":
             if char == '\\':
                 state = "read escaped char"
-            elif char == ',':
+            elif char == listSeparator:
                 state = "read normal"
                 results.append(current)
                 current = ""
@@ -1506,7 +1512,8 @@ def stringToTextValues(s, strict=False):
                     current = current + char
             else:
                 state = "read normal"
-                current = current + char #this is an error, but whatever
+                # leave unrecognized escaped characters for later passes
+                current = current + '\\' + char 
 
         elif state == "end":    #an end state
             if current != "" or len(results) == 0:
